verbose=False
dry=False

help_texts = {
    'location': 'Location to run code e.g docker/local',
    'force_all': 'Force all experiment to re run, otherwise will only run those that have not already completed',
    'filter': '',
    'filter_file': '',
    'observer': 'Run experiment with a sacred observer'
}

experiment_config = {}


